Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports Microsoft.SharePoint
Imports Microsoft.SharePoint.Taxonomy
Imports System.Web.UI.WebControls
Imports System.Web.UI
Imports Microsoft.SharePoint.Publishing

Namespace ArticleProject
	Public Class TaxonomyDataSource
		Inherits XmlDataSource
		Private Sub GetXmlFragment(ByVal publishingPageCollection As PublishingPageCollection, ByVal term As Term, ByRef xml As String)
			Dim url As String = ""
			Dim closeTerm As Boolean = False
			Dim defaultPublishingPage As PublishingPage = Nothing

			For Each publishingPage As PublishingPage In publishingPageCollection
				Dim values As TaxonomyFieldValueCollection = TryCast(publishingPage.ListItem("Tags"), TaxonomyFieldValueCollection)

				For Each value As TaxonomyFieldValue In values
					If value IsNot Nothing AndAlso value.TermGuid = term.Id.ToString() Then
						url = publishingPage.Uri.AbsoluteUri
						xml &= "<Term Name='" & term.Name & "' URL='" & url & "?CurrentTermId=" & term.Id.ToString() & "'>"
						closeTerm = True
						defaultPublishingPage = publishingPage
						Exit For
					End If
				Next value

				If closeTerm Then
					Exit For
				End If
			Next publishingPage

			For Each cterm As Term In term.Terms
				GetXmlFragment(publishingPageCollection, cterm, xml)
			Next cterm


			If closeTerm Then
				xml &= "</Term>"
			End If

			If Not Me.IsGlobal Then
				For Each publishingPage As PublishingPage In publishingPageCollection
					If publishingPage Is defaultPublishingPage Then
						Continue For
					End If

					Dim values As TaxonomyFieldValueCollection = TryCast(publishingPage.ListItem("Tags"), TaxonomyFieldValueCollection)
					For Each value As TaxonomyFieldValue In values
						If value IsNot Nothing AndAlso value.TermGuid = term.Id.ToString() Then
							url = publishingPage.Uri.AbsoluteUri
							xml &= "<Term Name='" & publishingPage.Title & "' URL='" & url & "?CurrentTermId=" & term.Id.ToString() & "'/>"
							Exit For
						End If
					Next value
				Next publishingPage
			End If
		End Sub

		Public Overrides Property Data() As String
			Get
				Dim publishingWeb As PublishingWeb = PublishingWeb.GetPublishingWeb(SPContext.Current.Web)
				Dim publishingPageCollection As PublishingPageCollection = publishingWeb.GetPublishingPages()

				Dim taxonomySession As New TaxonomySession(SPContext.Current.Site)
				Dim termsToReturn As TermCollection = Nothing

				Dim xml As String = "<Terms>"

				Dim values As TaxonomyFieldValueCollection = TryCast(SPContext.Current.ListItem("Tags"), TaxonomyFieldValueCollection)
				Dim currentTerm As Term = Nothing
				For Each value As TaxonomyFieldValue In values
					currentTerm = taxonomySession.GetTerm(New Guid(value.TermGuid))
					GetXmlFragment(publishingPageCollection, currentTerm, xml)
					termsToReturn = currentTerm.Terms

					If termsToReturn IsNot Nothing Then
						For Each term As Term In termsToReturn
							GetXmlFragment(publishingPageCollection, term, xml)
						Next term
					End If
				Next value

				xml &= "</Terms>"

				Return xml
			End Get
			Set(ByVal value As String)
				Throw New NotSupportedException("The Data property cannot be set!")
			End Set
		End Property

		Public Property TermStore() As String
			Get
				Return If(ViewState("TermStore") IsNot Nothing, CStr(ViewState("TermStore")), "")
			End Get
			Set(ByVal value As String)
				ViewState("TermStore") = value
			End Set
		End Property

		Public Property Group() As String
			Get
				Return If(ViewState("Group") IsNot Nothing, CStr(ViewState("Group")), "")
			End Get
			Set(ByVal value As String)
				ViewState("Group") = value
			End Set
		End Property

		Public Property TermSet() As String
			Get
				Return If(ViewState("TermSet") IsNot Nothing, CStr(ViewState("TermSet")), "")
			End Get
			Set(ByVal value As String)
				ViewState("TermSet") = value
			End Set
		End Property

		Public Property IsGlobal() As Boolean
			Get
				Return If(ViewState("IsGlobal") IsNot Nothing, CBool(ViewState("IsGlobal")), True)
			End Get
			Set(ByVal value As Boolean)
				ViewState("IsGlobal") = value
			End Set
		End Property
	End Class


End Namespace